/*
** Module   : CommonSocketListener.java
** Abstract : An interface holding the SERVER-SOCKET specific methods and 
**            attributes api.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 CS  20130204 Created initial version.
*/

package com.goldencode.p2j.util;

/**
 * Interface for SERVER-SOCKET methods and attributes
 * <p>
 * Exposes the following Attributes and methods:
 * <table>
 * <tr>
 *    <th>attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>ENABLE-CONNECTIONS</td>
 *    <td>{@link #enableConnections(String)}, 
 *        {@link #enableConnections(character)}
 *    </td>
 * </tr>
 * <tr>
 *    <td>DISABLE-CONNECTIONS</td>
 *    <td>{@link #disableConnections()}</td>
 * </tr>
 * <tr>
 *    <td>SET-CONNECT-PROCEDURE</td>
 *    <td>{@link #setConnectProcedure(String)},
 *        {@link #setConnectProcedure(character)},
 *        {@link #setConnectProcedure(String, handle) },
 *        {@link #setConnectProcedure(character, handle)},
 *        
 *    </td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapSocketListener(handle)} API, which, will return a
 * {@link SocketListenerImpl} for a Server-Socket type handle.
 */
public interface CommonSocketListener 
extends CommonHandleChain
{
   
   /**
    * Java implementation for the ENABLE-CONNECTION()  SERVER-SOCKET method, 
    * this will attempt to start listening  on the port specified by the 
    * option argument, and will return <code>true</code> if operation 
    * succeeded, or <code>false</code> otherwise.
    * <p>
    * The following options can be specified as valid ENABLE-CONNECTIONS() 
    * options:
    * <ul>
    *    <li><b>-S port</b> The port to which to connect.
    *    <li><b>-ssl</b> If specified the connection will be made using ssl.
    *    <li><b>-pf config-file-name</b> Specifies the file from which options
    *        will be taken.
    *    <li><b>-qsize maxconn</b> The maximum number of connections to handle 
    *        before refusing extra incoming connections.
    *    <li><b>-keyalias alias</b> The alias for the key and digital 
    *        certificates, default is : default_server.
    *   
    *    <li><b>-keyaliaspasswd pwd</b> The password for the key and digital
    *        certificate. To use always with -keyalias option.
    *    <li><b>-nossesioncache</b> Disables SSL client session cache.
    *    <li><b>-sessiontimeout time</b> The number of seconds before the SSL
    *        client request is timeout, default value is 180..
    * </ul>
    * 
    * @param  option
    *         The set of options that define the enable-connections method.
    * 
    * @return <code>true</code> if operation succeeded, or <code>false</code>
    *         otherwise
    */
   public logical enableConnections(String option);
   
   /**
    * Java implementation for the ENABLE-CONNECTION()  SERVER-SOCKET method, 
    * this will attempt to start listening  on the port specified by the 
    * option argument, and will return <code>true</code> if operation 
    * succeeded, or <code>false</code> otherwise.
    * <p>
    * The following options can be specified as valid ENABLE-CONNECTIONS() 
    * options:
    * <ul>
    *    <li><b>-S port</b> The port to which to connect.
    *    <li><b>-ssl</b> If specified the connection will be made using ssl.
    *    <li><b>-pf config-file-name</b> Specifies the file from which options
    *        will be taken.
    *    <li><b>-qsize maxconn</b> The maximum number of connections to handle 
    *        before refusing extra incoming connections.
    *    <li><b>-keyalias alias</b> The alias for the key and digital 
    *        certificates, default is : default_server.
    *   
    *    <li><b>-keyaliaspasswd pwd</b> The password for the key and digital
    *        certificate. To use always with -keyalias option.
    *    <li><b>-nossesioncache</b> Disables SSL client session cache.
    *    <li><b>-sessiontimeout time</b> The number of seconds before the SSL
    *        client request is timeout, default value is 180..
    * </ul>
    * 
    * @param  option
    *         The set of options that define the enable-connections method.
    * 
    * @return <code>true</code> if operation succeeded, or <code>false</code>
    *         otherwise
    */
   public logical enableConnections(character option);
   
   /**
    * Java implementation for the DISABLE-CONNECTIONS() SERVER-SOCKET method, 
    * this will stop listening and accepting any new connections, but without
    * interrupting the existing ones that haven't finished, and returns 
    * <code>true</code> if succeeded or  <code>false</code> otherwise.
    * 
    * @return <code>true</code> if the operation succeeded or <code>false</code> 
    *         otherwise.
    */
   public logical disableConnections();
   
   /**
    * Java implementation for the SET-CONNECT-PROCEDURE() SERVER-SOCKET method, 
    * this will set the procedure to execute when a CONNECT event is 
    * received.
    * <p>
    * The set-connect-procedure can also contain a handle to a higher 
    * level procedure which contains the read response procedure (to resolve 
    * any procedure context issues).
    * 
    * @param  procedureName
    *         The name of the procedure we want to set for read response.
    *         
    * @return <code>true</code> if the operation was successful or 
    *         <code>false</code> otherwise.
    */
   public logical setConnectProcedure(String procedureName);
   
   /**
    * Java implementation for the SET-CONNECT-PROCEDURE() SERVER-SOCKET method, 
    * this will set the procedure to execute when a CONNECT event is 
    * received.
    * <p>
    * The set-connect-procedure can also contain a handle to a higher 
    * level procedure which contains the read response procedure (to resolve 
    * any procedure context issues).
    * 
    * @param  procedureName
    *         The name of the procedure we want to set for read response.
    *         
    * @return <code>true</code> if the operation was successful or 
    *         <code>false</code> otherwise.
    */
   public logical setConnectProcedure(character procedureName);
   
   /**
    * Java implementation for the SET-CONNECT-PROCEDURE() SERVER-SOCKET method, 
    * this will set the procedure to execute when a CONNECT event is 
    * received.
    * <p>
    * The set-connect-procedure can also contain a handle to a higher 
    * level procedure which contains the read response procedure (to resolve 
    * any procedure context issues).
    * 
    * @param  procedureName
    *         The name of the procedure we want to set for read response.
    * @param  procHandle
    *         The handle for the procedure containing the procedure we want
    *         to set as connect procedure.Optional
    *         
    * @return <code>true</code> if the operation was successful or 
    *         <code>false</code> otherwise.
    */
   public logical setConnectProcedure(String procedureName, handle procHandle);
   
   /**
    * Java implementation for the SET-CONNECT-PROCEDURE() SERVER-SOCKET method, 
    * this will set the procedure to execute when a CONNECT event is 
    * received.
    * <p>
    * The set-connect-procedure can also contain a handle to a higher 
    * level procedure which contains the read response procedure (to resolve 
    * any procedure context issues).
    * 
    * @param  procedureName
    *         The name of the procedure we want to set for read response.
    * @param  procHandle
    *         The handle for the procedure containing the procedure we want
    *         to set as connect procedure.Optional
    *         
    * @return <code>true</code> if the operation was successful or 
    *         <code>false</code> otherwise.
    */
   public logical setConnectProcedure(character procedureName,
                                      handle procHandle);
}
